/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikacolorbutton.h
 * Copyright (C) 1999-2001 Sven Neumann
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

/* This provides a button with a color preview. The preview
 * can handle transparency by showing the checkerboard.
 * On click, a color selector is opened, which is already
 * fully functional wired to the preview button.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_COLOR_BUTTON_H__
#define __PIKA_COLOR_BUTTON_H__

#include <libpikawidgets/pikabutton.h>

G_BEGIN_DECLS


#define PIKA_TYPE_COLOR_BUTTON            (pika_color_button_get_type ())
#define PIKA_COLOR_BUTTON(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_BUTTON, PikaColorButton))
#define PIKA_COLOR_BUTTON_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_BUTTON, PikaColorButtonClass))
#define PIKA_IS_COLOR_BUTTON(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_BUTTON))
#define PIKA_IS_COLOR_BUTTON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_BUTTON))
#define PIKA_COLOR_BUTTON_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_BUTTON, PikaColorButtonClass))


typedef struct _PikaColorButtonPrivate PikaColorButtonPrivate;
typedef struct _PikaColorButtonClass   PikaColorButtonClass;

struct _PikaColorButton
{
  PikaButton              parent_instance;

  PikaColorButtonPrivate *priv;
};

struct _PikaColorButtonClass
{
  PikaButtonClass  parent_class;

  /*  signals  */
  void  (* color_changed)   (PikaColorButton *button);

  /*  virtual functions  */
  GType (* get_action_type) (PikaColorButton *button);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType          pika_color_button_get_type         (void) G_GNUC_CONST;

GtkWidget    * pika_color_button_new              (const gchar       *title,
                                                   gint               width,
                                                   gint               height,
                                                   const PikaRGB     *color,
                                                   PikaColorAreaType  type);

void           pika_color_button_set_title        (PikaColorButton   *button,
                                                   const gchar       *title);
const gchar  * pika_color_button_get_title        (PikaColorButton   *button);

void           pika_color_button_set_color        (PikaColorButton   *button,
                                                   const PikaRGB     *color);
void           pika_color_button_get_color        (PikaColorButton   *button,
                                                   PikaRGB           *color);

gboolean       pika_color_button_has_alpha        (PikaColorButton   *button);
void           pika_color_button_set_type         (PikaColorButton   *button,
                                                   PikaColorAreaType  type);

gboolean       pika_color_button_get_update       (PikaColorButton   *button);
void           pika_color_button_set_update       (PikaColorButton   *button,
                                                   gboolean           continuous);

void           pika_color_button_set_color_config (PikaColorButton   *button,
                                                   PikaColorConfig   *config);

GSimpleActionGroup * pika_color_button_get_action_group   (PikaColorButton   *button);


G_END_DECLS

#endif /* __PIKA_COLOR_BUTTON_H__ */
